@@ -30,6 +30,9 @@ FITS2NUMPY = {'L': 'i1', 'B': 'u1', 'I': 'i2', 'J': 'i4', 'K': 'i8', 'E': 'f4',
 
 # the inverse dictionary of the above
 NUMPY2FITS = dict([(val, key) for key, val in FITS2NUMPY.iteritems()])
+# Normally booleans are represented as ints in pyfits, but if passed in a numpy
+# boolean array, that should be supported
+NUMPY2FITS['b1'] = 'L'
 
 # lists of column/field definition common names and keyword names, make
 # sure to preserve the one-to-one correspondence when updating the list(s).
@@ -200,12 +203,6 @@ class Column(object):
 
         # scale the array back to storage values if there is bscale/bzero
         if isinstance(array, np.ndarray):
-
-            # boolean needs to be scaled too
-            if recfmt[-2:] == FITS2NUMPY['L']:
-                _out = np.zeros(array.shape, dtype=recfmt)
-                array = np.where(array==0, ord('F'), ord('T'))
-
             # make a copy if scaled, so as not to corrupt the original array
             if bzero not in ['', None, 0] or bscale not in ['', None, 1]:
                 array = array.copy()
@@ -247,6 +244,12 @@ class Column(object):
                 else:
                     numpy_format = _convert_format(format)
                     return _convert_array(array, np.dtype(numpy_format))
+            elif 'L' in format:
+                # boolean needs to be scaled back to storage values ('T', 'F')
+                if array.dtype == np.dtype('bool'):
+                    return np.where(array == False, ord('F'), ord('T'))
+                else:
+                    return np.where(array == 0, ord('F'), ord('T'))
             elif 'X' not in format and 'P' not in format:
                 (repeat, fmt, option) = _parse_tformat(format)
                 # Preserve byte order of the original array for now; see #77
@@ -375,26 +378,11 @@ class ColDefs(object):
 
             # now build the columns
             self.columns = [Column(**attrs) for attrs in col_attributes]
-            self._listener = input
+            self._listener = weakref.proxy(input)
         else:
             raise TypeError('Input to ColDefs must be a table HDU or a list '
                             'of Columns.')
 
-        # For ASCII tables, reconstruct string columns and ensure that spaces
-        # are used for padding instead of \x00, and do the reverse for binary
-        # table columns.
-        for col in self.columns:
-            array = col.array
-            if not isinstance(array, chararray.chararray):
-                continue
-            for i in range(len(array)):
-                al = len(array[i])
-                if isinstance(array[i], unicode):
-                    pad = self._padding_byte
-                else:
-                    pad = self._padding_byte.encode('ascii')
-                array[i] = array[i] + (pad * (array.itemsize - al))
-
     def __getattr__(self, name):
         """
         Automatically returns the values for the given keyword attribute for
@@ -481,9 +469,12 @@ class ColDefs(object):
 
     def _update_listener(self):
         if hasattr(self, '_listener'):
-            if self._listener._data_loaded:
-                del self._listener.data
-            self._listener.columns = self
+            try:
+                if self._listener._data_loaded:
+                    del self._listener.data
+                self._listener.columns = self
+            except ReferenceError:
+                del self._listener
 
     def add_col(self, column):
         """
