@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# $Id: core.py 1037 2011-08-23 21:23:12Z stsci_embray $
+# $Id: core.py 1423 2012-03-29 13:47:59Z stsci_embray $
 
 """
 A module for reading and writing FITS files and manipulating their
@@ -48,7 +48,6 @@ from pyfits.convenience import *
 from pyfits.fitsrec import FITS_record, FITS_rec
 from pyfits.hdu import *
 
-from pyfits.hdu.groups import GroupData
 from pyfits.hdu.hdulist import fitsopen as open
 from pyfits.hdu.image import Section
 from pyfits.hdu.table import new_table
@@ -60,11 +59,11 @@ from pyfits.header import Header
 from pyfits.verify import VerifyError
 
 
-__all__ = pyfits.card.__all__ + pyfits.column.__all__ + \
-          pyfits.convenience.__all__ + pyfits.hdu.__all__ + \
-          ['FITS_record', 'FITS_rec', 'GroupData', 'open', 'Section',
-           'new_table', 'Header', 'VerifyError', 'TRUE', 'FALSE',
-           'setExtensionNameCaseSensitive']
+__all__ = (pyfits.card.__all__ + pyfits.column.__all__ +
+           pyfits.convenience.__all__ + pyfits.hdu.__all__ +
+           ['FITS_record', 'FITS_rec', 'open', 'Section', 'new_table',
+            'Header', 'VerifyError', 'TRUE', 'FALSE',
+            'setExtensionNameCaseSensitive'])
 
 
 # These are of course deprecated, but a handful of external code still uses
