@@ -13,9 +13,10 @@ import pyfits.core
 from pyfits.card import Card, _pad
 from pyfits.file import _File
 from pyfits.header import Header
-from pyfits.util import Extendable, _with_extensions, lazyproperty, _is_int, \
-                        _is_pseudo_unsigned, _unsigned_zero, _pad_length, \
-                        itersubclasses, decode_ascii, BLOCK_SIZE, deprecated
+from pyfits.util import (Extendable, _with_extensions, lazyproperty, _is_int,
+                         _is_pseudo_unsigned, _unsigned_zero, _pad_length,
+                         itersubclasses, decode_ascii, BLOCK_SIZE, deprecated,
+                         _get_array_memmap)
 from pyfits.verify import _Verify, _ErrList
 
 
@@ -105,6 +106,7 @@ class _BaseHDU(object):
         self._hdrLoc = None
         self._datLoc = None
         self._datSpan = None
+        self._new = True
         self.name = ''
 
     def _getheader(self):
@@ -205,7 +207,6 @@ class _BaseHDU(object):
 
         hdu = cls(data=data, header=header, **new_kwargs)
 
-        size = hdu.size()
         hdu._file = fileobj
         hdu._hdrLoc = offset                 # beginning of the header area
         if fileobj:
@@ -214,6 +215,7 @@ class _BaseHDU(object):
             hdu._datLoc = hdrlen
 
         # data area size, including padding
+        size = hdu.size()
         hdu._datSpan = size + _pad_length(size)
 
         # Checksums are not checked on invalid HDU types
@@ -265,8 +267,17 @@ class _BaseHDU(object):
         # continue reading header blocks until END card is reached
         while True:
             # find the END card
-            mo = HEADER_END_RE.search(block)
-            if mo is None:
+            is_end = False
+            for mo in HEADER_END_RE.finditer(block):
+                # Ensure the END card was found, and it started on the
+                # boundary of a new card (see ticket #142)
+                if mo.start() % Card.length == 0:
+                    # This must be the last header block, otherwise the
+                    # file is malformatted
+                    is_end = True
+                    break
+
+            if not is_end:
                 blocks.append(block)
                 block = decode_ascii(fileobj.read(BLOCK_SIZE))
                 if block == '':
@@ -308,8 +319,9 @@ class _BaseHDU(object):
         fileobj.seek(hdu._datLoc + hdu._datSpan, os.SEEK_SET)
         return hdu
 
-    def _writeheader(self, fileobj, checksum=False):
-        # NOTE: Right now this assumes fileobj is a _File object
+    # TODO: Rework checksum handling so that it's not necessary to add a
+    # checksum argument here
+    def _prewriteto(self, checksum=False, inplace=False):
         # If the data is unsigned int 16, 32, or 64 add BSCALE/BZERO
         # cards to header
         # I'm thinking maybe this code belongs in the ImageBaseHDU and/or
@@ -324,24 +336,43 @@ class _BaseHDU(object):
                                 after='BSCALE')
 
         # Handle checksum
-        if 'CHECKSUM' in self._header:
-            del self._header['CHECKSUM']
 
-        if 'DATASUM' in self._header:
-            del self._header['DATASUM']
+        # If the data is loaded it isn't necessarily 'modified', but we have no
+        # way of knowing for sure
+        modified = self._header._mod or self._data_loaded
 
-        if checksum == 'datasum':
-            self.add_datasum()
-        elif checksum == 'nonstandard_datasum':
-            self.add_datasum(blocking='nonstandard')
-        elif checksum == 'test':
-            self.add_datasum(self._datasum_comment)
-            self.add_checksum(self._checksum_comment, True)
-        elif checksum == 'nonstandard':
-            self.add_checksum(blocking='nonstandard')
-        elif checksum:
-            self.add_checksum(blocking='standard')
+        if modified or (not checksum and not inplace):
+            if 'CHECKSUM' in self._header:
+                del self._header['CHECKSUM']
 
+            if 'DATASUM' in self._header:
+                del self._header['DATASUM']
+
+        if checksum and (modified or self._new):
+            if checksum == 'datasum':
+                self.add_datasum()
+            elif checksum == 'nonstandard_datasum':
+                self.add_datasum(blocking='nonstandard')
+            elif checksum == 'test':
+                self.add_datasum(self._datasum_comment)
+                self.add_checksum(self._checksum_comment, True)
+            elif checksum == 'nonstandard':
+                self.add_checksum(blocking='nonstandard')
+            elif checksum:
+                self.add_checksum(blocking='standard')
+
+    def _postwriteto(self):
+        # If data is unsigned integer 16, 32 or 64, remove the
+        # BSCALE/BZERO cards
+        if (self._data_loaded and self.data is not None and
+            self._standard and _is_pseudo_unsigned(self.data.dtype)):
+            for keyword in ('BSCALE', 'BZERO'):
+                try:
+                    del self._header[keyword]
+                except KeyError:
+                    pass
+
+    def _writeheader(self, fileobj):
         blocks = repr(self._header.ascard) + _pad('END')
         blocks = blocks + _pad_length(len(blocks)) * ' '
 
@@ -361,12 +392,8 @@ class _BaseHDU(object):
             fileobj.write(blocks.encode('ascii'))
             fileobj.flush()
 
-        # If data is unsigned integer 16, 32 or 64, remove the
-        # BSCALE/BZERO cards
-        if (self._data_loaded and self.data is not None and
-            self._standard and _is_pseudo_unsigned(self.data.dtype)):
-            del self._header['BSCALE']
-            del self._header['BZERO']
+        # Update hdrLoc with the new offset
+        self._hdrLoc = offset
 
         return offset, size
 
@@ -398,8 +425,12 @@ class _BaseHDU(object):
         if not fileobj.simulateonly:
             fileobj.flush()
 
+        # Update datLoc with the new offset
+        self._datLoc = offset
+        self._datSpan = size = size + _pad_length(size)
+
         # return both the location and the size of the data area
-        return offset, size + _pad_length(size)
+        return offset, size
 
     def _writedata_internal(self, fileobj):
         """
@@ -418,10 +449,64 @@ class _BaseHDU(object):
     # Though right now this is an internal private method (though still used by
     # HDUList, eventually the plan is to have this be moved into writeto()
     # somehow...
-    def _writeto(self, fileobj, checksum=False):
+    def _writeto(self, fileobj, inplace=False, copy=False):
         # For now fileobj is assumed to be a _File object
-        return (self._writeheader(fileobj, checksum)[0],) + \
-               self._writedata(fileobj)
+        if not inplace or self._new:
+            self._writeheader(fileobj)
+            self._writedata(fileobj)
+            return
+
+        hdrloc = self._hdrLoc
+        hdrsize = self._datLoc - self._hdrLoc
+        datloc = self._datLoc
+        datsize = self._datSpan
+
+        if self.header._mod:
+            # Seek to the original header location in the file
+            self._file.seek(hdrloc)
+            # This should update hdrloc with he header location in the new file
+            hdrloc, hdrsize = self._writeheader(fileobj)
+
+            # If the data is to be written below with self._writedata, that
+            # will also properly update the data location; but it should be
+            # updated here too
+            datloc = hdrloc + hdrsize
+        elif copy:
+            # Seek to the original header location in the file
+            self._file.seek(hdrloc)
+            # Before writing, update the hdrloc with the current file position,
+            # which is the hdrloc for the new file
+            hdrloc = fileobj.tell()
+            fileobj.write(self._file.read(hdrsize))
+            # The header size is unchanged, but the data location may be
+            # different from before depending on if previous HDUs were resized
+            datloc = fileobj.tell()
+        if self._data_loaded:
+            if self.data is not None:
+                # Seek through the array's bases for an memmap'd array; we
+                # can't rely on the _File object to give us this info since the
+                # user may have replaced the previous mmap'd array
+                if copy:
+                    # Of course, if we're copying the data to a new file we
+                    # don't care about flushing the original mmap; instead just
+                    # read it into the new file
+                    memmap_array = None
+                else:
+                    memmap_array = _get_array_memmap(self.data)
+
+                if memmap_array is not None:
+                    memmap_array.flush()
+                else:
+                    self._file.seek(self._datLoc)
+                    datloc, datsize = self._writedata(fileobj)
+        elif copy:
+            # Seek to the data location in the original file
+            self._file.seek(self._datLoc)
+            fileobj.write(self._file.read(datsize))
+
+        self._hdrLoc = hdrloc
+        self._datLoc = datloc
+        self._datSpan = datsize + _pad_length(datsize)
 
     @_with_extensions
     def writeto(self, name, output_verify='exception', clobber=False,
@@ -583,8 +668,7 @@ class _NonstandardHDU(_BaseHDU, _Verify):
         Return the file data.
         """
 
-        self._file.seek(self._datLoc)
-        return self._file.readarray()
+        return self._file.readarray(self.size(), self._datLoc)
 
     def _verify(self, option='warn'):
         errs = _ErrList([], unit='Card')
@@ -1426,8 +1510,7 @@ class NonstandardExtHDU(ExtensionHDU):
         Return the file data.
         """
 
-        self._file.seek(self._datLoc)
-        return self._file.readarray()
+        return self._file.readarray(self.size(), self._datLoc)
 # TODO: Mark this as deprecated
 _NonstandardExtHDU = NonstandardExtHDU
 
