@@ -274,6 +274,28 @@ class TestCore(PyfitsTestCase):
         assert_equal(hdu.name, 'SCI')
         assert_equal(hdu.header['EXTNAME'], 'SCI')
 
+    def test_nonstandard_hdu(self):
+        """
+        Regression test for #157.  Tests that "Nonstandard" HDUs with SIMPLE =
+        F are read and written without prepending a superfluous and unwanted
+        standard primary HDU.
+        """
+
+        data = np.arange(100, dtype=np.uint8)
+        hdu = pyfits.PrimaryHDU(data=data)
+        hdu.header['SIMPLE'] = False
+        hdu.writeto(self.temp('test.fits'))
+
+        info = [(0, '', 'NonstandardHDU', 5, (), '', '')]
+        with pyfits.open(self.temp('test.fits')) as hdul:
+            assert_equal(hdul.info(output=False), info)
+            # NonstandardHDUs just treat the data as an unspecified array of
+            # bytes.  The first 100 bytes should match the original data we
+            # passed in...the rest should be zeros padding out the rest of the
+            # FITS block
+            assert_true((hdul[0].data[:100] == data).all())
+            assert_true((hdul[0].data[100:] == 0).all())
+
 
 class TestConvenienceFunctions(PyfitsTestCase):
     def test_writeto(self):
