@@ -6,6 +6,7 @@ import os
 import numpy as np
 
 import pyfits
+from pyfits.verify import VerifyError
 from pyfits.tests import PyfitsTestCase
 from pyfits.tests.util import catch_warnings, BytesIO, ignore_warnings
 
@@ -53,6 +54,16 @@ class TestHDUListFunctions(PyfitsTestCase):
         res = hdul.fileinfo(2)
         test_fileinfo(resized=1, datLoc=17280, hdrLoc=11520)
 
+    def test_create_from_multiple_primary(self):
+        """
+        Regression test for #145.  Ensure that a validation error occurs when
+        saving an HDUList containing multiple PrimaryHDUs.
+        """
+
+        hdul = pyfits.HDUList([pyfits.PrimaryHDU(), pyfits.PrimaryHDU()])
+        assert_raises(VerifyError, hdul.writeto, self.temp('temp.fits'),
+                      output_verify='exception')
+
     def test_append_primary_to_empty_list(self):
         # Tests appending a Simple PrimaryHDU to an empty HDUList.
         hdul = pyfits.HDUList()
@@ -503,3 +514,91 @@ class TestHDUListFunctions(PyfitsTestCase):
                 assert_equal(repr(hdul[0].header.ascard),
                              repr(hdu.header.ascard))
                 assert_true((hdul[0].data == a).all())
+
+    def test_update_with_truncated_header(self):
+        """
+        Regression test for #148.  Test that saving an update where the header
+        is shorter than the original header doesn't leave a stump from the old
+        header in the file.
+        """
+
+        data = np.arange(100)
+        hdu = pyfits.PrimaryHDU(data=data)
+        idx = 1
+        while len(hdu.header) < 34:
+            hdu.header.update('TEST%d' % idx, idx)
+            idx += 1
+        hdu.writeto(self.temp('temp.fits'), checksum=True)
+
+        with pyfits.open(self.temp('temp.fits'), mode='update') as hdul:
+            # Modify the header, forcing it to be rewritten
+            hdul[0].header['TEST1'] = 2
+
+        with pyfits.open(self.temp('temp.fits')) as hdul:
+            assert_true((hdul[0].data == data).all())
+
+    def test_update_resized_header(self):
+        """
+        Test saving updates to a file where the header is one block smaller
+        than before, and in the case where the header is one block larger than
+        before.
+        """
+
+        data = np.arange(100)
+        hdu = pyfits.PrimaryHDU(data=data)
+        idx = 1
+        while len(hdu.header) <= 35:
+            hdu.header.update('TEST%d' % idx, idx)
+            idx += 1
+        orig_header = hdu.header.copy()
+        hdu.writeto(self.temp('temp.fits'))
+
+        with pyfits.open(self.temp('temp.fits'), mode='update') as hdul:
+            while len(hdul[0].header) >= 36:
+                del hdul[0].header[-1]
+
+        with pyfits.open(self.temp('temp.fits')) as hdul:
+            assert_equal(str(hdul[0].header.ascard), str(orig_header[:-1]))
+            assert_true((hdul[0].data == data).all())
+
+        with pyfits.open(self.temp('temp.fits'), mode='update') as hdul:
+            idx = 101
+            while len(hdul[0].header) < 36 * 2:
+                hdul[0].header.update('TEST%d' % idx, idx)
+                idx += 1
+            # Touch something in the data too so that it has to be rewritten
+            hdul[0].data[0] = 27
+
+        with pyfits.open(self.temp('temp.fits')) as hdul:
+            assert_equal(str(hdul[0].header[:-37]), str(orig_header[:-1]))
+            assert_equal(hdul[0].data[0], 27)
+            assert_true((hdul[0].data[1:] == data[1:]).all())
+
+    def test_update_resized_header2(self):
+        """
+        Regression test for #150.  This is similar to
+        test_update_resized_header, but specifically tests a case of multiple
+        consecutive flush() calls on the same HDUList object, where each
+        flush() requires a resize.
+        """
+
+        data1 = np.arange(100)
+        data2 = np.arange(100) + 100
+        phdu = pyfits.PrimaryHDU(data=data1)
+        hdu = pyfits.ImageHDU(data=data2)
+        phdu.writeto(self.temp('temp.fits'))
+        with pyfits.open(self.temp('temp.fits'), mode='append') as hdul:
+            hdul.append(hdu)
+        with pyfits.open(self.temp('temp.fits'), mode='update') as hdul:
+            idx = 1
+            while len(hdul[0].header) < 36 * 2:
+                hdul[0].header.update('TEST%d' % idx, idx)
+                idx += 1
+            hdul.flush()
+            hdul.append(hdu)
+
+        with pyfits.open(self.temp('temp.fits')) as hdul:
+            assert_true((hdul[0].data == data1).all())
+            assert_equal(hdul[1].header, hdu.header)
+            assert_true((hdul[1].data == data2).all())
+            assert_true((hdul[2].data == data2).all())
