@@ -1,8 +1,10 @@
 import functools
 import itertools
 import os
+import signal
 import sys
 import tempfile
+import threading
 import warnings
 
 import numpy as np
@@ -11,8 +13,10 @@ import numpy as np
 __all__ = ['Extendable', 'register_extension', 'register_extensions',
            'unregister_extensions']
 
+from numpy import memmap as Memmap
 
-BLOCK_SIZE = 2880 # the FITS block size
+
+BLOCK_SIZE = 2880  # the FITS block size
 
 
 # TODO: I'm somewhat of the opinion that this should go in pyfits.core, but for
@@ -365,7 +369,7 @@ def deprecated(message='', name='', alternative='', pending=False):
                     _message += '  Use %(alternative)s instead.'
 
             if pending:
-                category = DeprecationPendingWarning
+                category = PendingDeprecationWarning
             else:
                 category = DeprecationWarning
 
@@ -384,6 +388,51 @@ def deprecated(message='', name='', alternative='', pending=False):
     return deprecate
 
 
+def ignore_sigint(func):
+    """
+    This decorator registers a custom SIGINT handler to catch and ignore SIGINT
+    until the wrapped function is completed.
+    """
+
+    @functools.wraps(func)
+    def wrapped(*args, **kwargs):
+        # Get the name of the current thread and determine if this is a single
+        # treaded application
+        curr_thread = threading.currentThread()
+        single_thread = (threading.activeCount() == 1 and
+                         curr_thread.getName() == 'MainThread')
+
+        class SigintHandler(object):
+            def __init__(self):
+                self.sigint_received = False
+
+            def __call__(self, signum, frame):
+                warnings.warn('KeyboardInterrupt ignored until %s is '
+                              'complete!' % func.__name__)
+                self.sigint_received = True
+
+        sigint_handler = SigintHandler()
+
+        # Define new signal interput handler
+        if single_thread:
+            # Install new handler
+            old_handler = signal.signal(signal.SIGINT, sigint_handler)
+
+        try:
+            func(*args, **kwargs)
+        finally:
+            if single_thread:
+                if old_handler is not None:
+                    signal.signal(signal.SIGINT, old_handler)
+                else:
+                    signal.signal(signal.SIGINT, signal.SIG_DFL)
+
+                if sigint_handler.sigint_received:
+                    raise KeyboardInterrupt
+
+    return wrapped
+
+
 def pairwise(iterable):
     """Return the items of an iterable paired with its next item.
 
@@ -472,6 +521,20 @@ def isfile(f):
     return isinstance(f, file)
 
 
+def fileobj_open(filename, mode):
+    """
+    A wrapper around the `open()` builtin.
+
+    This exists because in Python 3, `open()` returns an `io.BufferedReader` by
+    default.  This is bad, because `io.BufferedReader` doesn't support random
+    access, which we need in some cases.  In the Python 3 case (implemented in
+    the py3compat module) we must call open with buffering=0 to get a raw
+    random-access file reader.
+    """
+
+    return open(filename, mode)
+
+
 def fileobj_name(f):
     """
     Returns the 'name' of file-like object f, if it has anything that could be
@@ -553,6 +616,10 @@ def translate(s, table, deletechars):
         return s.translate(table)
 
 
+def indent(s, shift=1, width=4):
+    return '\n'.join(' ' * (width * shift) + l if l else ''
+                     for l in s.splitlines())
+
 
 def _array_from_file(infile, dtype, count, sep):
     """Create a numpy array from a file or a file-like object."""
@@ -721,3 +788,19 @@ def _tmp_name(input):
     f, fn = tempfile.mkstemp(dir=input)
     os.close(f)
     return fn
+
+
+def _get_array_memmap(array):
+    """
+    If the array has a numpy.memmap as one of its bases, return the memmap
+    base; otherwise return None.
+    """
+
+    if isinstance(array, Memmap):
+        return array
+
+    base = array
+    while hasattr(base, 'base') and base.base is not None:
+        if isinstance(base.base, Memmap):
+            return base.base
+        base = base.base
